#/opt/python3.10/bin/python3.10
# -*- coding: utf-8 -*-
import os
import re
import pymysql
import time
import json
import logging.config
import configparser
from artix.filequeue.filequeuemanager import FileQueueManager

logging.config.fileConfig('maint_log.ini')
progressDir = "/linuxcash/cash/exchangesystems/progress"
progressFileName = os.path.join(progressDir, "counter_maint_prices.json")
log = logging.getLogger('su.artix.pricedeleteservice')
configparser = configparser.RawConfigParser()
configparser.read("config.ini")
shop_type = configparser.get('MAIN', 'shop_type')
db_host = configparser.get('DATABASE', 'host')
db_port = configparser.getint('DATABASE', 'port')
db_user = configparser.get('DATABASE', 'user')
db_password = configparser.get('DATABASE', 'password')


def chunks(chunkable, n):
    for i in range(0, len(chunkable), n):
        yield chunkable[i:i + n]


def getDatabases():
    conn = pymysql.connect(host=db_host, port=db_port, user=db_user, passwd=db_password)
    cur = conn.cursor()
    cur.execute("show databases like '%%dictionaries%'")
    dbses = [db[0] for db in cur.fetchall()]
    cur.close()
    conn.close()
    for db in dbses:
        re_db = re.search('^.*(dictionariesAll)', db)
        if re_db:
            re_db = re_db.group(1)
        else: re_db = 'dictionariesAll'
        if shop_type == 'All' and db == re_db:
            log.info("Database: %s", db)
            yield db
        elif shop_type == 'shops' and db != re_db:
            log.info("Database: %s", db)
            yield db
        elif shop_type == 'both':
            log.info("Database: %s", db)
            yield db


def createTransaction(log, commandList, shopId, clusterId):
    if (shopId.lower() == 'all'):
        shopQueueName = clusterId + 'All' if clusterId else 'All'
    else:
        shopQueueName = shopId
    queueName = 'artix.loaddict.' + ((shopQueueName + '.' + clusterId) if clusterId else shopQueueName)
    #queueName = 'artix.loaddict.' + shopId
    try:
        fileQueueManager = FileQueueManager(queueRoot='/opt/queues', log=log, location='cs')
        fileQueueManager.removeUncompletedTransactions(queueName, 'maint')
        transactionId = fileQueueManager.beginTransaction(queueName)
        fileQueueManager.enqueue(queueName, transactionId, {'properties': {'type': 'datagram'}, 'content': commandList})
        endTransProperties = {
            'type': 'control',
            'control': 'end_tran',
            'statusServer': 'localhost:8080',
            'source': 'maint',
            'transactionId': transactionId,
            'objectsCount': len(commandList)
            }
        fileQueueManager.endTransaction(queueName, transactionId, {'properties': endTransProperties})
    except Exception as e:
        log.error('Ошибка: %s' % str(e))


def readJson(path):
    try:
        with open(path) as data_file:
            data = json.load(data_file)
        return data
    except Exception as e:
        raise Exception("Failed to read %s. %s" % (path, str(e)))


def get_shop_id(db):
    shopId = re.search('^.*dictionaries(.+)', db)
    if shopId:
        shopId = shopId.group(1)
    return shopId


def get_cluster_id(db):
    clusterId = re.search('(.+)dictionaries.+$', db)
    if clusterId:
        clusterId = clusterId.group(1)
    return clusterId


def calculateData():
    conn = None
    cur = None
    try:
        log.info('Удаляем старые переоценки')
        data = readJson('maint.json')
        resultCount = data['prices']['pricesCount'] if 'prices' in data and 'pricesCount' in data['prices'] else 500
        timeout = data['prices']['timeout'] if 'prices' in data and 'timeout' in data['prices'] else 24
        log.info('Начало выборки данных из бд')
        for db in getDatabases():
            shopId = get_shop_id(db)
            clusterId = get_cluster_id(db)
            conn = pymysql.connect(host=db_host, port=db_port, user=db_user, passwd=db_password)
            cur = conn.cursor()
            cur.execute('set names utf8')
            cur.execute("SELECT code FROM `%s`.tmc" % db)
            codeList = [item[0] for item in cur.fetchall()]
            log.info('Выбрали все коды из таблицы tmc')
            cur.execute("SELECT barcode FROM `%s`.barcodes" % db)
            tempList = [item[0] for item in cur.fetchall()]
            log.info('Выбрали все штрих-коды из таблицы barcodes')
            codeList.extend(tempList)
            log.info('Собрали два списка вместе')

            cur.execute("SELECT DISTINCT(barcode) FROM `%s`.prices WHERE effectivedateend < now()" % db)
            result = [{'command': 'deleteOverduePrices', 'barcode': str(item[0])} for item in cur.fetchall()]
            for data in chunks(result, resultCount):
                log.info('Посылаем %d команд в nes' % len(data))
                createTransaction(log, data, shopId, clusterId)
                time.sleep(timeout)

            commandList = []
            for row in chunks(codeList, 50):
                row = filter(None, row)
                query_values = ', '.join("'%s'" % it for it in row)
                query = "SELECT barcode, max(effectivedate), effectivedateend FROM `%s`.prices \
                        WHERE barcode IN (%s) AND effectivedate < now() AND (effectivedateend > now() OR effectivedateend IS NULL) \
                        GROUP BY barcode, effectivedateend HAVING count(barcode) > 1" % (db, str(query_values))
                cur.execute(query)
                result = [
                    {'command': 'deletePriceByDate',
                     'barcode': str(item[0]),
                     'effectivedate': str(item[1]),
                     'effectivedateend': str(item[2]) if item[2] else None}
                    for item in cur.fetchall()]

                if result:
                    log.info(str(result))
                    commandList.extend(result)
                    log.info('Сформировали %d команд для nes. Общий размер пакета %d' % (len(result), len(commandList)))

                    if len(commandList) >= resultCount:
                        for data in chunks(commandList, resultCount):
                            log.info('Посылаем %d команд в nes' % len(data))
                            createTransaction(log, data, shopId, clusterId)
                        commandList = []
            if commandList:
                createTransaction(log, commandList, shopId, clusterId)
    except Exception as e:
        log.error('Ошибка : %s' % e)
    finally:
        if conn and conn.open:
            if cur:
                cur.close()
            conn.close()


def getTPricesDeleteCommand(cur, db, tindex, bcode):
    """
    Получить команду для на удаление для пары bcode и tindex
    :param cur: курсор
    :param tindex: индекс
    :param bcode: штрих-код
    :return: команда на удаление
    """
    # Выбираем ближаюшую к текущей прошедшую дату effectivedate для пары bcode и tindex
    cur.execute('SELECT effectivedate FROM `%s`.tprices '
                'WHERE effectivedate < now() AND bcode = "%s" AND tindex = %d '
                'ORDER BY effectivedate DESC LIMIT 1' % (db, bcode, tindex))
    item = cur.fetchone()
    # После удаления должна остаться одна запись, так что удаляем все записи с датой меньше выбранной
    return {'command': 'deleteOutdatedAdditionalPrice',
            'pricecode': tindex,
            'barcode': bcode,
            'effectivedate': str(item[0])}


def removeOverdueAdditionalPrices():
    """
    Удалить старые доп. цены из таблицы tprices, с сохранением одной записи для каждой пары bcode и tindex
    """
    conn = None
    cur = None
    try:
        log.info('Удаляем старые доп. цены')
        conf = readJson('maint.json')
        resultCount = conf['prices']['pricesCount'] if 'prices' in conf and 'pricesCount' in conf['prices'] else 500
        for db in getDatabases():
            shopId = get_shop_id(db)
            clusterId = get_cluster_id(db)
            conn = pymysql.connect(host=db_host, port=db_port, user=db_user, passwd=db_password)
            cur = conn.cursor()
            cur.execute('set names utf8')
            log.info("Выбраем пары tindex и bcode для удаления устаревших доп. цен")
            # Ищем уникальные пары bcode и tindex с количеством записей больше 1
            query = "SELECT tindex, bcode FROM `%s`.tprices \
                    WHERE effectivedate < now() GROUP BY tindex, bcode HAVING COUNT(*) > 1" % db
            cur.execute(query)
            log.info("Формируем список команд на удаления доп. цен")
            commands = [getTPricesDeleteCommand(cur, db, int(item[0]), str(item[1])) for item in cur.fetchall()]
            log.info("Список команд сформирован")
            for cmds in chunks(commands, resultCount):
                log.info('Посылаем %d команд в nes' % len(cmds))
                createTransaction(log, cmds, shopId, clusterId)
    except Exception as e:
        log.error('Ошибка : %s' % e)
    finally:
        if conn and conn.open:
            if cur:
                cur.close()
            conn.close()


def main():
    log.info('Сервис maint_prices начал работу')
    calculateData()
    removeOverdueAdditionalPrices()
    log.info('Сервис maint_prices закончил работу')


if __name__ == '__main__':
    main()
